@@ -8,8 +8,6 @@
     :copyright: (c) 2010 by Armin Ronacher, Georg Brandl.
     :license: BSD, see LICENSE for more details.
 """
-from __future__ import with_statement
-
 import threading
 from itertools import count
 from thread import get_ident as current_thread
@@ -38,6 +36,62 @@ def group_reflected_property(name, default, fallback=_missing):
     return property(_get, _set, _del)
 
 
+class _StackBound(object):
+
+    def __init__(self, obj, push, pop):
+        self.__obj = obj
+        self.__push = push
+        self.__pop = pop
+
+    def __enter__(self):
+        self.__push()
+        return self.__obj
+
+    def __exit__(self, exc_type, exc_value, tb):
+        self.__pop()
+
+
+class StackedObject(object):
+    """Baseclass for all objects that provide stack manipulation
+    operations.
+    """
+
+    def push_thread(self):
+        """Pushes the stacked object to the thread stack."""
+        raise NotImplementedError()
+
+    def pop_thread(self):
+        """Pops the stacked object from the thread stack."""
+        raise NotImplementedError()
+
+    def push_application(self):
+        """Pushes the stacked object to the application stack."""
+        raise NotImplementedError()
+
+    def pop_application(self):
+        """Pops the stacked object from the application stack."""
+        raise NotImplementedError()
+
+    def __enter__(self):
+        self.push_thread()
+        return self
+
+    def __exit__(self, exc_type, exc_value, tb):
+        self.pop_thread()
+
+    def threadbound(self, _cls=_StackBound):
+        """Can be used in combination with the `with` statement to
+        execute code while the object is bound to the thread.
+        """
+        return _cls(self, self.push_thread, self.pop_thread)
+
+    def applicationbound(self, _cls=_StackBound):
+        """Can be used in combination with the `with` statement to
+        execute code while the object is bound to the application.
+        """
+        return _cls(self, self.push_application, self.pop_application)
+
+
 class ContextStackManager(object):
     """Helper class for context objects that manages a stack of
     objects.
@@ -67,7 +121,8 @@ class ContextStackManager(object):
         return iter(objects)
 
     def push_thread(self, obj):
-        with self._context_lock:
+        self._context_lock.acquire()
+        try:
             self._cache.pop(current_thread(), None)
             item = (self._stackop(), obj)
             stack = getattr(self._context, 'stack', None)
@@ -75,13 +130,18 @@ class ContextStackManager(object):
                 self._context.stack = [item]
             else:
                 stack.append(item)
+        finally:
+            self._context_lock.release()
 
     def pop_thread(self):
-        with self._context_lock:
+        self._context_lock.acquire()
+        try:
             self._cache.pop(current_thread(), None)
             stack = getattr(self._context, 'stack', None)
             assert stack, 'no objects on stack'
             return stack.pop()[1]
+        finally:
+            self._context_lock.release()
 
     def push_application(self, obj):
         self._global.append((self._stackop(), obj))
